/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2010, Digium, Inc.
 *
 * Tilghman Lesher <tlesher AT digium DOT com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*!
 * \file
 * \brief Function FILE tests
 *
 * \author\verbatim Tilghman Lesher <tlesher AT digium DOT com> \endverbatim
 *
 * \ingroup tests
 */

/*** MODULEINFO
	<depend>TEST_FRAMEWORK</depend>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 262852 $")

#include "asterisk/utils.h"
#include "asterisk/app.h"
#include "asterisk/module.h"
#include "asterisk/test.h"
#include "asterisk/pbx.h"

static struct {
	const char *contents;
	const char *args;
	const char *value;
} read_tests[] = {
	/* 4 different ways of specifying the first character */
	{ "123456789", "0,1", "1" },
	{ "123456789", "0,-8", "1" },
	{ "123456789", "-9,1", "1" },
	{ "123456789", "-9,-8", "1" },
	/* Does 0-length work? */
	{ "123456789", "0,0", "" },
	{ "123456789", "-9,0", "" },
	{ "123456789", "-9,-9", "" },
	/* Does negative length work? */
	{ "123456789", "5,-6", "" },
	{ "123456789", "-5,-6", "" },
	/* No length */
	{ "123456789", "-5", "56789" },
	{ "123456789", "4", "56789" },
	/* Line mode, 4 ways of specifying the first character */
	{ "123\n456\n789\n", "L0,L1", "123\n" },
	{ "123\n456\n789\n", "L-3,L1", "123\n" },
	{ "123\n456\n789\n", "L0,L-2", "123\n" },
	{ "123\n456\n789\n", "L-3,L-2", "123\n" },
	/* Line mode, 0-length */
	{ "123\n456\n789\n", "L0,L0", "" },
	{ "123\n456\n789\n", "L-3,L0", "" },
	{ "123\n456\n789\n", "L-3,L-3", "" },
	/* Line mode, negative length */
	{ "123\n456\n789\n", "L2,L-2", "" },
	{ "123\n456\n789\n", "L-2,L-3", "" },
	/* No length */
	{ "123\n456\n789\n", "L1", "456\n789\n" },
	{ "123\n456\n789\n", "L-2", "456\n789\n" },
};

static struct {
	const char *contents;
	const char *args;
	const char *value;
	const char *contents2;
} write_tests[] = {
	/* Single character replace */
	{ "123456789", "0,1", "a", "a23456789" },
	{ "123456789", "-9,1", "a", "a23456789" },
	{ "123456789", "0,-8", "a", "a23456789" },
	{ "123456789", "-9,-8", "a", "a23456789" },
	{ "123456789", "5,1", "b", "12345b789" },
	{ "123456789", "-4,1", "b", "12345b789" },
	{ "123456789", "5,-3", "b", "12345b789" },
	{ "123456789", "-4,-3", "b", "12345b789" },
	/* Replace 2 characters with 1 */
	{ "123456789", "0,2", "c", "c3456789" },
	{ "123456789", "-9,2", "c", "c3456789" },
	{ "123456789", "0,-7", "c", "c3456789" },
	{ "123456789", "-9,-7", "c", "c3456789" },
	{ "123456789", "4,2", "d", "1234d789" },
	{ "123456789", "-5,2", "d", "1234d789" },
	{ "123456789", "4,-3", "d", "1234d789" },
	{ "123456789", "-5,-3", "d", "1234d789" },
	/* Truncate file */
	{ "123456789", "5", "e", "12345e" },
	{ "123456789", "5", "", "12345" },
	{ "123456789", "-4", "e", "12345e" },
	{ "123456789", "-4", "", "12345" },
	/* Replace 1 character with 2 */
	{ "123456789", "0,1", "fg", "fg23456789" },
	{ "123456789", "0,-8", "fg", "fg23456789" },
	{ "123456789", "-9,1", "fg", "fg23456789" },
	{ "123456789", "-9,-8", "fg", "fg23456789" },
};

AST_TEST_DEFINE(test_func_file)
{
	int res = AST_TEST_PASS;
	int i;
	char dir[] = "/tmp/test_func_file.XXXXXX";
	char file[80], expression[256];
	struct ast_str *buf;
	char fbuf[256];
	FILE *fh;

	switch (cmd) {
	case TEST_INIT:
		info->name = "func_file";
		info->category = "funcs/func_env";
		info->summary = "Verify behavior of the FILE() dialplan function";
		info->description =
			"Verifies that the examples of the FILE() dialplan function documentation work as described.";
		return AST_TEST_NOT_RUN;
	case TEST_EXECUTE:
		break;
	}

	if (!mkdtemp(dir)) {
		ast_test_status_update(test, "Cannot create temporary directory: %s\n", strerror(errno));
		return AST_TEST_FAIL;
	}

	if (!(buf = ast_str_create(16))) {
		rmdir(dir);
		return AST_TEST_FAIL;
	}

	snprintf(file, sizeof(file), "%s/test.txt", dir);

	for (i = 0; i < ARRAY_LEN(read_tests); i++) {
		if (!(fh = fopen(file, "w"))) {
			ast_test_status_update(test, "Cannot open test file: %s\n", strerror(errno));
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		if (fwrite(read_tests[i].contents, 1, strlen(read_tests[i].contents), fh) < strlen(read_tests[i].contents)) {
			ast_test_status_update(test, "Cannot write initial values into test file: %s\n", strerror(errno));
			fclose(fh);
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		fclose(fh);

		snprintf(expression, sizeof(expression), "${FILE(%s,%s)}", file, read_tests[i].args);
		ast_str_reset(buf);
		ast_str_substitute_variables(&buf, 0, NULL, expression);

		if (strcmp(ast_str_buffer(buf), read_tests[i].value)) {
			if (strchr(read_tests[i].value, '\n')) {
				ast_test_status_update(test, "Expression '%s' did not produce the expected value\n", expression);
			} else {
				ast_test_status_update(test, "Expression '%s' did not produce ('%s') the expected value ('%s')\n", expression, ast_str_buffer(buf), read_tests[i].value);
			}
			res = AST_TEST_FAIL;
		}
	}

	for (i = 0; i < ARRAY_LEN(write_tests); i++) {
		if (!(fh = fopen(file, "w"))) {
			ast_test_status_update(test, "Cannot open test file: %s\n", strerror(errno));
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		if (fwrite(write_tests[i].contents, 1, strlen(write_tests[i].contents), fh) < strlen(write_tests[i].contents)) {
			ast_test_status_update(test, "Cannot write initial values into test file: %s\n", strerror(errno));
			fclose(fh);
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		fclose(fh);

		snprintf(expression, sizeof(expression), "FILE(%s,%s)", file, write_tests[i].args);
		ast_func_write(NULL, expression, write_tests[i].value);

		if (!(fh = fopen(file, "r"))) {
			ast_test_status_update(test, "Cannot open test file: %s\n", strerror(errno));
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		if (!fread(fbuf, 1, sizeof(fbuf), fh)) {
			ast_test_status_update(test, "Cannot read write results from test file: %s\n", strerror(errno));
			fclose(fh);
			unlink(file);
			rmdir(dir);
			return AST_TEST_FAIL;
		}

		fclose(fh);

		if (strcmp(fbuf, write_tests[i].contents2)) {
			if (strchr(write_tests[i].contents2, '\n')) {
				ast_test_status_update(test, "Expression '%s=%s' did not produce a file with the expected result\n", expression, write_tests[i].value);
			} else {
				ast_test_status_update(test, "Expression '%s=%s' did not produce ('%s') the expected value ('%s')\n",
					expression, write_tests[i].value, ast_str_buffer(buf), write_tests[i].contents2);
			}
			res = AST_TEST_FAIL;
		}
	}

	unlink(file);
	rmdir(dir);

	return res;
}

static int unload_module(void)
{
	AST_TEST_UNREGISTER(test_func_file);
	return 0;
}

static int load_module(void)
{
	AST_TEST_REGISTER(test_func_file);
	return AST_MODULE_LOAD_SUCCESS;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "FILE() Tests");
